import path from 'path'
import fs from 'fs'
import * as hogan from 'hogan.js'
import defaultTemplate from './defaultTemplate'

interface package2mdField {
    examples?: string[],
    description?: string
}



class ModifiedPackage {
    static _modules: { [key: string]: ModifiedPackage } = {}
    private _path: string
    package2md: package2mdField = {
        examples: []
    }
    repository: { url: string, type: string }
    name: string
    author: string
    dependencies: ModifiedPackage[]
    devDependencies: ModifiedPackage[]

    static getPackage(packagePath) {
        return ModifiedPackage._modules[packagePath] || new ModifiedPackage(packagePath)
    }
    private modifiers = {
        examples:(pkg)=>{
            let ret=[]
            if (pkg.examples && Array.isArray(pkg.examples)) {
                ret = pkg.examples.map((filePath: string) => {
                    return fs.existsSync(filePath) ? fs.readFileSync(filePath).toString() : null
                }).filter(n => n)
            }
            return ret
        },
        dependencies: (pkg) => {
            return Object.keys(pkg.dependencies || {}).map((key) => {
                const dependencyPath = path.resolve(this._path, 'node_modules', key)
                if (fs.existsSync(dependencyPath)) {
                    return ModifiedPackage.getPackage(dependencyPath)
                }
                return null
            }).filter(n => n)
        },
        devDependencies: (pkg) => {
            return Object.keys(pkg.devDependencies || {}).map((key) => {
                const dependencyPath = path.resolve(this._path, 'node_modules', key)
                if (fs.existsSync(dependencyPath)) {
                    return ModifiedPackage.getPackage(dependencyPath)
                }
                return null
            }).filter(n => n)
        },
        repository: (pkg) => {
            if ('string' === typeof pkg.repository) {
                const arr = pkg.repository.split(/\:\//g, 1)
                return { type: arr[0], url: arr[1] }
            }
            return pkg.repository
        }
    }

    /**
     * 
     * @param {string} packagePath : the package path
     */
    constructor(packagePath: string) {
        console.log('packagePath::::::::', packagePath)
        const t = ModifiedPackage._modules[packagePath] = this
        t._path = packagePath
        let pkg: { [key: string]: any }
        try {
            pkg = JSON.parse(fs.readFileSync(path.join(packagePath, 'package.json')).toString())
        } catch (e) {
            throw new Error(`Unable to parse ${packagePath}: ${e.message}`)
        }

        try{
            pkg=Object.assign({},pkg,JSON.parse(fs.readFileSync(path.join(packagePath, 'package2md.json')).toString()))
        }catch(_e){}

        Object.keys(t.modifiers).forEach(k => {
            pkg[k] = t.modifiers[k](pkg)
        })

        Object.keys(pkg).forEach(k => {
            t[k] = pkg[k]
        })
    }
}
export function createReadme(packagePath: string, template: string = defaultTemplate) {
    const pkg = ModifiedPackage.getPackage(packagePath)
    console.log(Object.keys(ModifiedPackage._modules))
    fs.writeFileSync(path.join(packagePath, 'README.md'), hogan.compile(template).render(pkg))
}
